import 'package:flutter/widgets.dart';

import 'list_listenable.dart';

typedef ListWidgetBuilder<T> = Widget Function(
    BuildContext context, T value, int index, Widget? child);

class ListListenableBuilder<T> extends StatefulWidget {
  /// Creates a [ValueListenableBuilder].
  ///
  /// The [listListenable] and [builder] arguments must not be null.
  /// The [child] is optional but is good practice to use if part of the widget
  /// subtree does not depend on the value of the [listListenable].
  const ListListenableBuilder({
    Key? key,
    required this.listListenable,
    required this.builder,
    this.child,
  })  : assert(listListenable != null),
        assert(builder != null),
        super(key: key);

  final ListListenable<T> listListenable;

  final ListWidgetBuilder<T> builder;

  final Widget? child;

  @override
  State<StatefulWidget> createState() => _ValueListenableBuilderState<T>();
}

class _ValueListenableBuilderState<T> extends State<ListListenableBuilder<T>> {
  late List<T> list;

  @override
  void initState() {
    super.initState();
    list = widget.listListenable.list;
    widget.listListenable.addListener(_valueChanged);
  }

  @override
  void didUpdateWidget(ListListenableBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.listListenable != widget.listListenable) {
      oldWidget.listListenable.removeListener(_valueChanged);
      list = widget.listListenable.list;
      widget.listListenable.addListener(_valueChanged);
    }
  }

  @override
  void dispose() {
    widget.listListenable.removeListener(_valueChanged);
    super.dispose();
  }

  void _valueChanged() {
    setState(() {
      list = widget.listListenable.list;
    });
  }

  @override
  Widget build(BuildContext context) {
    return ListView.builder(
        itemBuilder: (context, index) =>
            widget.builder(context, list[index], index, widget.child),
      itemCount: list.length
    );
  }
}
